#!/usr/bin/php -q
<?php
# $Id: command_def_fix.php,v 1.1 2004/10/26 15:34:55 exon Exp $
#
# Quick-hack script to fix various things between upgrades from Nagios 1.2
# to Nagios 2.0. It's rather brutish, and has a limited understanding of
# the configuration file syntax, but it should be able to do most of the
# dirty work pretty quickly even for the largest of configs.
#

$comments = false;
$obj_num = false;

$Config['main'] = $argv[1];

/* swallows objects from a single object configuration file in non-static
 * style. Slow, but precision is more important.
 * Now also keeps the top-level comments */
function import_objects($fname) {
	global $comments;
	
	$inobj = false;
	$objs = 0;
	$in_topcomment = true;
	$topcomment = '';

	if(!($buf = file_get_contents($fname))) return false;
	$lines = explode("\n", $buf);

	foreach($lines as $line) {
		/* Skip comments and 'out of object' data, unless we're
		 * still working on the topcomments(syntax explanation) */
		$line = trim($line);
		if(!strlen($line)) {
			$in_topcomment = false;
			continue;
		}

		if($line{0} === '#') {
			if(!$in_topcomment) continue;
			if(!isset($comments[$fname])) $comments[$fname] = "$line\n";
			else $comments[$fname] .= "$line\n";
			continue;
		}
		else $in_topcomment = false;

		// add object type
		if(!strncmp($line, 'define', strlen('define'))) {
			$str = split("[\t {]+", $line, 3);
			$ret_array[$objs]['otype'] = $str[1];
		}

		if(strchr($line, '{')) {
			$inobj = true;
			continue;
		}
		if(strchr($line, '}')) {
			$inobj = false;
			$objs++;
			continue;
		}
		if($inobj === false) continue;

		/* separate token and insert */
		$str = split("[\t ]+", $line, 2);
		$ret_array[$objs][$str[0]] = $str[1];
	}

	if(!isset($ret_array)) $ret_array = false;

	return($ret_array);
}

/*
 * Get configuration files from main config file and return them in an array.
 */
function get_config_files() {
	global $Config;

	$cfg_files = array();

	// read, mangle and put in array
	$buf = file_get_contents($Config['main']);
	$lines = explode("\n", $buf);
	foreach($lines as $line) {
		// skip junk, split string and evaluate
		if(!strlen($line) || $line[0] === '#') continue;
		$str = split("[\t ]*=[\t ]*", trim($line), 2);
		if($str[0] == "cfg_file") {
			$cfg_files[] = $str[1];
		}
		// found a dir, so get files from it(recursive?)
		else if($str[0] == "cfg_dir") {
			$dir_files = glob($str[1] . "/*.cfg");
			foreach($dir_files as $df) {
				$cfg_files[] = $str[1] . "/" . $df;
			}
		}
	}

	return($cfg_files);
}

/*
 * import objects from config files(uses get_config_files)
 * All objects are sorted as per object_type here. Merging, splitting and
 * cross-referencing objects is done someplace else.
 */
function import_object_configuration() {
	global $Config;
	global $F;
	$obj_count = false;

	$cfg_files = get_config_files();

	foreach($cfg_files as $cfg_file) {
		$cfg_array[$cfg_file] = import_objects($cfg_file);
	}

	foreach($cfg_array as $cfg_file => $obj_list) {
		if(!is_array($obj_list)) continue;
		foreach($obj_list as $obj) {
			$otype = $obj['otype'];
			if(!isset($obj_count[$otype])) $obj_count[$otype] = 0;
			unset($obj['otype']);
			$obj['FILE'] = $cfg_file;
			$obj_array[$otype][] = $obj;
		}
	}

	return($obj_array);
}


$obj_array = import_object_configuration();

$host_cg = false;

$check_list = array('host', 'contact', 'command', 'service', 'hostgroup');
foreach($check_list as $cl) {
	if(empty($obj_array[$cl])) {
		echo "\n\n\n################################################################\n";
		echo "No $cl objects configured (somethings very wrong with that)\n";
		echo "Exiting to prevent configuration damage\n";
		echo "################################################################\n\n\n";
		exit(1);
	}
}

# replace obsolete notification commands
$obj_list = $obj_array['contact'];
foreach($obj_list as $obj_key => $obj) {
	if($obj['service_notification_commands'] === 'notify-by-sms' ||
	   $obj['service_notification_commands'] === 'notify-by-email') {
		$obj['service_notification_commands'] = 'service-notify';
		echo "\tReplacing obsolete service_notification_commands for user $obj[contact_name]\n";
	}
	if($obj['host_notification_commands'] === 'host-notify-by-email' ||
	   $obj['host_notification_commands'] === 'notify-by-sms') {
		echo "\tReplacing obsolete host_notification_commands for user $obj[contact_name]\n";
		$obj['host_notification_commands'] = 'host-notify';
	}
	$obj_list[$obj_key] = $obj;
}
$obj_array['contact'] = $obj_list;

# remove obsolete notification commands and fix host-notify and service-notify
$obj_list = $obj_array['command'];
foreach($obj_list as $obj_key => $obj) {
	if($obj['command_name'] === 'service-notify') {
		echo "\tUpdating old service-notify command object definition\n";
		$obj['command_line'] = '$USER3$/notify/notify -c "$CONTACTNAME$" -h "$HOSTNAME$" -f "$NOTIFICATIONTYPE$" -m "$CONTACTEMAIL$" -p "$CONTACTPAGER$" -s "$SERVICEDESC$" "HOSTALIAS=$HOSTALIAS$" "HOSTADDRESS=$HOSTADDRESS$" "HOSTSTATE=$HOSTSTATE$" "HOSTSTATEID=$HOSTSTATEID$" "HOSTSTATETYPE=$HOSTSTATETYPE$" "HOSTATTEMPT=$HOSTATTEMPT$" "HOSTLATENCY=$HOSTLATENCY$" "HOSTEXECUTIONTIME=$HOSTEXECUTIONTIME$" "HOSTDURATION=$HOSTDURATION$" "HOSTDURATIONSEC=$HOSTDURATIONSEC$" "HOSTDOWNTIME=$HOSTDOWNTIME$" "HOSTPERCENTCHANGE=$HOSTPERCENTCHANGE$" "HOSTGROUPNAME=$HOSTGROUPNAME$" "HOSTGROUPALIAS=$HOSTGROUPALIAS$" "LASTHOSTCHECK=$LASTHOSTCHECK$" "LASTHOSTSTATECHANGE=$LASTHOSTSTATECHANGE$" "LASTHOSTUP=$LASTHOSTUP$" "LASTHOSTDOWN=$LASTHOSTDOWN$" "LASTHOSTUNREACHABLE=$LASTHOSTUNREACHABLE$" "HOSTOUTPUT=$HOSTOUTPUT$" "HOSTPERFDATA=$HOSTPERFDATA$" "SERVICESTATE=$SERVICESTATE$" "SERVICESTATEID=$SERVICESTATEID$" "SERVICESTATETYPE=$SERVICESTATETYPE$" "SERVICEATTEMPT=$SERVICEATTEMPT$" "SERVICELATENCY=$SERVICELATENCY$" "SERVICEEXECUTIONTIME=$SERVICEEXECUTIONTIME$" "SERVICEDURATION=$SERVICEDURATION$" "SERVICEDURATIONSEC=$SERVICEDURATIONSEC$" "SERVICEDOWNTIME=$SERVICEDOWNTIME$" "SERVICEPERCENTCHANGE=$SERVICEPERCENTCHANGE$" "SERVICEGROUPNAME=$SERVICEGROUPNAME$" "SERVICEGROUPALIAS=$SERVICEGROUPALIAS$" "LASTSERVICECHECK=$LASTSERVICECHECK$" "LASTSERVICESTATECHANGE=$LASTSERVICESTATECHANGE$" "LASTSERVICEOK=$LASTSERVICEOK$" "LASTSERVICEWARNING=$LASTSERVICEWARNING$" "LASTSERVICEUNKNOWN=$LASTSERVICEUNKNOWN$" "LASTSERVICECRITICAL=$LASTSERVICECRITICAL$" "SERVICEOUTPUT=$SERVICEOUTPUT$" "SERVICEPERFDATA=$SERVICEPERFDATA$" "SERVICEACKAUTHOR=$SERVICEACKAUTHOR$" "SERVICEACKCOMMENT=$SERVICEACKCOMMENT$" "NOTIFICATIONNUMBER=$NOTIFICATIONNUMBER$" "SHORTDATETIME=$SHORTDATETIME$" "TIME=$TIME$" "TIMET=$TIMET$" "HOSTACTIONURL=$HOSTACTIONURL$" "HOSTNOTESURL=$HOSTNOTESURL$" "SERVICEACTIONURL=$SERVICEACTIONURL$" "SERVICENOTESURL=$SERVICENOTESURL$" "ADMINPAGER=$ADMINPAGER$" "ADMINEMAIL=$ADMINEMAIL$"';
	}
	elseif($obj['command_name'] === 'host-notify') {
		echo "\tUpdating old host-notify command object definition\n";
		$obj['command_line'] = '$USER3$/notify/notify -c "$CONTACTNAME$" -h "$HOSTNAME$" -f "$NOTIFICATIONTYPE$" -m "$CONTACTEMAIL$" -p "$CONTACTPAGER$" "HOSTALIAS=$HOSTALIAS$" "HOSTADDRESS=$HOSTADDRESS$" "HOSTSTATE=$HOSTSTATE$" "HOSTSTATEID=$HOSTSTATEID$" "HOSTSTATETYPE=$HOSTSTATETYPE$" "HOSTATTEMPT=$HOSTATTEMPT$" "HOSTLATENCY=$HOSTLATENCY$" "HOSTEXECUTIONTIME=$HOSTEXECUTIONTIME$" "HOSTDURATION=$HOSTDURATION$" "HOSTDURATIONSEC=$HOSTDURATIONSEC$" "HOSTDOWNTIME=$HOSTDOWNTIME$" "HOSTPERCENTCHANGE=$HOSTPERCENTCHANGE$" "HOSTGROUPNAME=$HOSTGROUPNAME$" "HOSTGROUPALIAS=$HOSTGROUPALIAS$" "LASTHOSTCHECK=$LASTHOSTCHECK$" "LASTHOSTSTATECHANGE=$LASTHOSTSTATECHANGE$" "LASTHOSTUP=$LASTHOSTUP$" "LASTHOSTDOWN=$LASTHOSTDOWN$" "LASTHOSTUNREACHABLE=$LASTHOSTUNREACHABLE$" "HOSTOUTPUT=$HOSTOUTPUT$" "HOSTPERFDATA=$HOSTPERFDATA$" "HOSTACKAUTHOR=$HOSTACKAUTHOR$" "HOSTACKCOMMENT=$HOSTACKCOMMENT$" "NOTIFICATIONNUMBER=$NOTIFICATIONNUMBER$" "CONTACTALIAS=$CONTACTALIAS$" "DATETIME=$DATETIME$" "SHORTDATETIME=$SHORTDATETIME$" "DATE=$DATE$" "TIME=$TIME$" "TIMET=$TIMET$" "HOSTACTIONURL=$HOSTACTIONURL$" "HOSTNOTESURL=$HOSTNOTESURL$" "ADMINPAGER=$ADMINPAGER$" "ADMINEMAIL=$ADMINEMAIL$"';
	}
	elseif($obj['command_name'] === 'host-notify-by-email' ||
		   $obj['command_name'] === 'notify-by-email' ||
		   $obj['command_name'] === 'notify-by-sms') {
		unset($obj_list[$obj_key]);
		continue;
	}
	$obj_list[$obj_key] = $obj;
}
$obj_array['command'] = $obj_list;

$config = false;
foreach($obj_array as $obj_type => $obj_list) {
	foreach($obj_list as $obj_key => $obj) {
		$f = $obj['FILE'];
		$obj['otype'] = $obj_type;
		unset($obj['FILE']);
		$config[$f][] = $obj;
	}
}
foreach($config as $cfg_file => $obj_list) {
	$fbuf[$cfg_file] = '';
	if(isset($comments[$cfg_file])) $fbuf[$cfg_file] = $comments[$cfg_file] . "\n\n";
	foreach($obj_list as $obj) {
		$obj_type = $obj['otype'];
		unset($obj['otype']);
		$fbuf[$cfg_file] .= "define $obj_type {\n";
		foreach($obj as $k => $v) {
			$fbuf[$cfg_file] .= sprintf("    %-30s %s\n", $k, $v);
		}
		$fbuf[$cfg_file] .= "    }\n\n";
	}
}
foreach($fbuf as $cfg_file => $buf) {
	$fh = fopen($cfg_file, "w");
	if(fwrite($fh, $buf) === false) {
		echo "Failed to write to file $cfg_file !!!\n";
	}
	else echo "Wrote file $cfg_file\n";
}

?>
