#!/usr/bin/perl -w
#
# $Id: notify.pl,v 1.14 2005/05/03 10:00:31 peter Exp $
#
# notify.pl - Notification script for OP5 Monitor
#
# Author: Andreas Ericsson <ae@op5.se>
#
# Copyright(C) 2004 OP5 AB
# All rights reserved.
#

use POSIX;
use strict;
use File::Glob ':glob';
use File::Basename;
use YAML::Syck;

# Http-post addition
use HTTP::Request; ####
use LWP::UserAgent; ###
use HTTP::Request::Common; ###

sub get_configparam {
	my ($key, $context, $default) = @_;
	my $configfile_path = "/etc/op5/$context.yml";
	my $config;
	if (-r $configfile_path and -f $configfile_path) {
		$config = YAML::Syck::LoadFile($configfile_path);
	}
	if (ref($config) and exists($config->{"$key"})) {
		return $config->{"$key"};
	} else {
		return $default;
	}
}

our $url_base = "https";
my $base_dir = dirname(__FILE__);
if (! -d $base_dir) {
	$base_dir = '/opt/monitor/op5/notify';
}

if ( -e "$base_dir/notify.inc.pl" ) {
	require "$base_dir/notify.inc.pl";
}

# set some defaults.
my ($sec, $min, $hour, $mday, $month, $year, $wday, $yday, $is_dst);
my $i;
my ($contact_pager, $contact_email, $host, $skin_file, $contact_name, $service, $notification_type, $contact_httppost);
my (%val, $k, $v, $from_email);
my $skin_dir = $base_dir;
my $custom_skin_dir = "$base_dir/custom-skins";
my $debug = 0;	# Use one (or more) -d flags to the scripts to enable
my $debug_handle;
my $x_var_dir = "$base_dir/extra_host_vars";
my @x_var_files;
my %x_vars;
my $foo;
my $result;
my $now;

# set up the status URL
my $hostname = get_configparam("hostname", "notify");
# try to get the system's hostname if it was not set in config
if (! length ($hostname // '')) {
	$hostname = `hostname -f` or $hostname = `hostname`;
	chomp($hostname);
}

sub logit {
	my $level = shift @_;
	my $msg = shift @_;
	my @caller_data = caller 1;
	my $caller;
	if (defined $caller_data[3]) {
		$caller = $caller_data[3];
	} else {
		$caller = caller;
	}
	if($debug < $level) {
		return;
	}
	if (!defined $debug_handle) {
		$debug_handle = *STDOUT;
	}
	$msg = "$caller: $msg\n";
	print {$debug_handle} $msg;
}

sub email_notify();
sub pager_notify();
sub httppost_notify(); # http-post addition
sub usage();

# --- Functions (ok subprocedures then, you silly, silly language)
$SIG{'ALRM'} = sub {
	print("Notification timed out\n");
	exit(1);
};

sub find_skin {
	my $default_file = "host.default";
	$service and $default_file = "service.default";

	my $method = shift @_;
	my @possible_dirs = (
		# this is undocumented, but the code always used it if $custom_skin_dir existed
		"$custom_skin_dir/skins.$method/$contact_name",
		"$skin_dir/skins.$method/$contact_name",
		"$custom_skin_dir/skins.$method",
		"$skin_dir/skins.$method"
	);
	my @possible_files = (
		$skin_file,
		$default_file
	);

	SKIN: foreach my $pos_dir (@possible_dirs) {
		if (! (-d $pos_dir)) {
			next;
		}
		foreach my $pos_file (@possible_files) {
			my $pos = "$pos_dir/$pos_file";
			if (! (-f $pos || -l $pos)) {
				next;
			}
			if (!-r $pos) {
				logit(1, "$pos isn't readable, so can't use that - dropping notification");
				return;
			}
			logit(2, "skinfile is $pos");
			return $pos;
		}
	}
	logit(1, "found no skin file to use, giving up");
	return;
};

sub pager_notify() {
	my $buf = undef;
	my $file = find_skin("sms");
	if (! $file) {
		return;
	}

	my $pager = '/usr/bin/smssend';
	if(!-x $pager) {
		$pager = '/usr/bin/sendsms';
	}

	logit(2, "using $pager to send sms");

	$buf .= `cat $file 2> /dev/null`;
	$? != 0 and return;

	foreach $k (keys %val) {
		$buf =~ s/#$k#/$val{$k}/g;
	}

	open(PIPE, "| $pager $contact_pager");
	print(PIPE "$buf\n");
	close(PIPE);

	logit(3, "$buf\n-------------------------------\n$file\n===========================\n");
};

# send an email, using the mail (or mailx command)
sub email_notify() {
	my $buf = undef;
	my $file = find_skin("mail");
	if (! $file) {
		return;
	}

	my $html_path = "/monitor";;
	my $status_url = "$url_base://";
	my $mailer = '/usr/sbin/sendmail -t -i';
	my $ack_url;

	$from_email and $mailer .= " -f$from_email";
	$status_url .= $hostname;
	$val{"SERVERADDRESS"} = $status_url;

	$status_url .= $html_path;

#	$status_url .= "/cgi-bin/status.cgi?host=$host";
	$status_url .= "/index.php/status/service/" . urlencode("$host");
	$val{"STATUS_URL"} = $status_url;

	$ack_url = "$url_base://" . $hostname . $html_path;
#	$ack_url .= '/cgi-bin/cmd.cgi?cmd_typ=';
	$ack_url .= '/index.php/command/submit?cmd_typ=';
#	https://dev-mon.int.op5.se/ninja/index.php/command/submit?cmd_typ=ACKNOWLEDGE_SVC_PROBLEM&host_name=build-centos-x86_64&service=Disk+usage+%2F
	if ($service) {
#		$ack_url .= '34&host=' . $host . '&service=' . $service;
		$ack_url .= 'ACKNOWLEDGE_SVC_PROBLEM&host_name=' . $host . '&service=' . $service;
	}
	else {
#		$ack_url .= '33&host=' . $host;
		$ack_url .= 'ACKNOWLEDGE_HOST_PROBLEM&host_name=' . $host;
	}
	$ack_url =~ s/([^-+=_A-Za-z0-9\/:.?&])/sprintf("%%%02X", ord($1))/seg;
	$val{"ACK_URL"} = $ack_url;

	logit(2, "Using $mailer to send mail");
	logit(2, "status_url = $status_url");

	$buf = `cat $file 2> /dev/null`;

	# read extra vars, if any
	my $x_vars = '';
	if(-d "$x_var_dir/$contact_name") {
		chdir("$x_var_dir/$contact_name");
		@x_var_files = glob('*');
		logit(2, $#x_var_files + 1 . " extra vars found in $x_var_dir/$contact_name");
		if($#x_var_files != -1) {
			for($i=0; $i<=$#x_var_files; $i++) {
				logit(2, "$i = $x_var_files[$i]");
				$foo = `grep ^$host $x_var_files[$i] | sed 's/^.*://'` and
				  $x_vars .= $x_var_files[$i] . '=' . $foo;
			}
		}
	}
	$val{"extra_host_vars"} = $x_vars;

	# -- get status url

	foreach $k (keys %val) {
		$buf =~ s/#$k#/$val{$k}/g;
	}

	# everything is set, so pipe to the mailer program with extra newline
	open(PIPE, "| $mailer");
	print(PIPE "$buf\n");
	close(PIPE);

	logit(3, "$buf\n-------------------------------\n$file\n=====================================");
};


# Post xml stuff to web server
sub httppost_notify() {
	my $buf = undef;
	my $file = find_skin("htmlpost");
	if (! $file) {
		return;
	}

	my $html_path = "/monitor";;
	my $status_url = "$url_base://";
	#   my $mailer = '/usr/sbin/sendmail -t -i';


	$status_url .= $hostname;

	$status_url .= $html_path;

	$status_url .= "/cgi-bin/status.cgi?host=$host";
	$val{"STATUS_URL"} = $status_url;

	logit(2,  "status_url = $status_url");

	$buf = `cat $file 2> /dev/null`;

	# read extra vars, if any
	my $x_vars = '';
	if(-d "$x_var_dir/$contact_name") {
		chdir("$x_var_dir/$contact_name");
		@x_var_files = glob('*');
		logit(2, $#x_var_files + 1 . " extra vars found in $x_var_dir/$contact_name");
		if($#x_var_files != -1) {
			for($i=0; $i<=$#x_var_files; $i++) {
				logit(2, "$i = $x_var_files[$i]");
				$foo = `grep ^$host $x_var_files[$i] | sed 's/^.*://'` and
				$x_vars .= $x_var_files[$i] . '=' . $foo;
			}
		}
	}
	$val{"extra_host_vars"} = $x_vars;


	# -- get status url

	foreach $k (keys %val) {
		$buf =~ s/#$k#/$val{$k}/g;
	}

	logit(3, "\n====================================\nPosting xml to: $contact_httppost\n------------------------------------\n");
	logit(3, "$buf\n-----------------------------------\n");

    my $contact_httppost = "" . $contact_httppost;
    my $content = $buf;

    my $header = HTTP::Headers->new(Content_Type=>'text/xml',
		                                    Content_Length=>length($content));
    my $request = HTTP::Request->new('POST', $contact_httppost,
		                                     $header, $content);


    my $ua = LWP::UserAgent->new;

    my $response = $ua->request($request);

	if($response->is_success){
		logit(2, "Post successful.\nResponse from webserver: $response->{_content}");
	} else {
		logit(1, "Post failed.\nResponse from webserver: $response->{_content}");
	}
};



sub usage() {
	print("Usage: notify.pl options variables...\n");
	print("\nWhere options are any or all of the following ('*' = required)\n\n");
	print("* -c | --contact      short contactname\n");
	print("  -m | --mail         contact email address. If blank no mail is sent\n");
	print("  -p | --pager        contact pager number. If blank no sms is sent\n");
	print("  -g | --httppost     post xml data to http server. If blank no post is done\n");
	print("* -h | --host         host this notification pertains to\n");
	print("  -s | --service      service_description (MUST be set for service notifications)\n");
	print("* -f | --skin         skin file (usually \$NOTIFICATIONTYPE\$)\n");
	print("  -d | --debug [file] Print debug info, optionally with output file (or stdout). Can be repeated\n");
	print("  -F | --from         set 'From: ' email address on mail notifications\n");
	print("\n");
	print("- Freeform variables should be specified as 'VARIABLE=VALUE'.\n");
	print("- Variables mentioned in the skin file as #VARIABLE# will be\n");
	print("  replaced with its corresponding VALUE.\n");
	print("- The reserved variables HOSTNAME and STATUS_URL are always available.\n");
	print("- The reserved variable SERVICEDESC is available if -s is non-empty\n");
	print("- Unix timestamps past the unix billenium will be translated to ISO 8601 format.\n");
	print "\n";
	exit(1);
};

# --- main code


$now = time();
`echo $now >> /tmp/notify_timestamps`;

# parse arguments
if($#ARGV < 2) {
	usage();
}

$val{"FROMEMAIL"} = 'op5Monitor';
for($i=0; $i<=$#ARGV; $i++) {
	if($ARGV[$i] =~ /^-m|^--mail|^--email/) {
		$contact_email = $ARGV[++$i];
		$val{"CONTACTEMAIL"} = $ARGV[$i];
	}
	elsif($ARGV[$i] =~ /^-F|^--from|^--from-email/) {
		$from_email = $ARGV[++$i];
		$val{"FROMEMAIL"} = $ARGV[$i];
	}
	elsif($ARGV[$i] =~ /^-p|^--page/) {
		$contact_pager = $ARGV[++$i];
		$val{"CONTACTPAGER"} = $ARGV[$i];
	}
	elsif($ARGV[$i] =~ /^-g|^--httppost/) {
		$contact_httppost = $ARGV[++$i];
		$val{"CONTACTHTMLPOST"} = $ARGV[$i];
	}
	elsif($ARGV[$i] =~ /^--help$/) {
		usage();
	}
	elsif($ARGV[$i] =~ /^-h|^--host/) {
		$host = $ARGV[++$i];
		$val{"HOSTNAME"} = $ARGV[$i];
	}
	elsif($ARGV[$i] =~ /^-u|^--user|^-c|^--contact/) {
		$contact_name = $ARGV[++$i];
		$val{"CONTACTNAME"} = $ARGV[$i];
	}
	elsif($ARGV[$i] =~ /^-f|^--skin/) {
		$skin_file = $ARGV[++$i];
		$notification_type = $ARGV[$i];
		$val{"NOTIFICATIONTYPE"} = $ARGV[$i];
	}
	elsif($ARGV[$i] =~ /^-d|^--debug/) {
		$debug++;
		if ($i + 1 <= $#ARGV && substr($ARGV[$i + 1], 0, 1) ne '-' && $ARGV[$i+1] !~ /=/) {
			open($debug_handle, '>'.$ARGV[++$i]);
		}
	}
	elsif($ARGV[$i] =~ /^-s|^--service/) {
		$service = $ARGV[++$i];
		$val{"SERVICEDESC"} = $ARGV[$i];
	}
	elsif($ARGV[$i] =~ /^(CONTACTADDRESS[0-6])=(.*)$/) {
		# we special-case CONTACTADDRESS macros, because they
		# frequently contain phonenumbers that invariably are
		# large enough to trigger the "assume timestamp" code
		$k = $1;
		$v = $2;
		$val{$k} = $v;
	}
	elsif($ARGV[$i] =~ /^([^=]*)=(.*)$/) {
		# this is a free form variable, so register it
		$k = $1;
		$v = $2;
		if($v && $v =~ /^\d*$/ && $v > 1000000000) {
			# number past the unix billenia. Assume timestamp and translate
			($sec, $min, $hour, $mday, $month, $year, $wday, $yday, $is_dst) = localtime($v);
			$v = sprintf("%4d-%02d-%02d %02d:%02d:%02d",$year+1900,$month+1,$mday,$hour,$min,$sec);
		}
		$val{$k} = $v;
	}
	else {
		print "Unknown option: " . $ARGV[$i] . "\n";
		usage();
	}
}

alarm(3);

$contact_name or $contact_name = "NO CONTACT DEFINED";
foreach $k (keys %val) {
	logit(2, "$k=" . $val{$k});
}

$host or die("Must set host parameter");

$skin_file or die("Must set skin file parameter");

# use a proper skin-file
if($service) {
	$skin_file = "service.$skin_file";
}
else {
	$skin_file = "host.$skin_file";
}

# -- take action based on means of transportations supplied
if($contact_pager && $contact_pager =~ /.*\d*.*/) {
	logit(1, "Sending pager notification to $contact_pager");
	pager_notify();
}
else {
	logit(2, "NOT sending pager notification");
}

if($contact_email && $contact_email =~ /^.*$/) {
	logit(1, "Sending email notification to $contact_email");
	email_notify();
}
else {
	logit(2, "NOT sending EMAIL notification");
}

if($contact_httppost && $contact_httppost =~ /^.*$/) {
	logit(1, "Posting notification to $contact_httppost");
	httppost_notify();
}
else {
	logit(2, "NOT posting xml notification");
}

sub urlencode {
	my $s = shift;
	$s =~ s/([^A-Za-z0-9\-])/sprintf("%%%02X", ord($1))/seg;
	return $s;
}
